@@ -26,24 +26,39 @@ G_BEGIN_DECLS
 
 #include "hdy-version.h"
 #include "hdy-action-row.h"
-#include "hdy-arrows.h"
-#include "hdy-column.h"
+#include "hdy-animation.h"
+#include "hdy-application-window.h"
+#include "hdy-avatar.h"
+#include "hdy-carousel.h"
+#include "hdy-clamp.h"
 #include "hdy-combo-row.h"
-#include "hdy-dialer-button.h"
-#include "hdy-dialer-cycle-button.h"
-#include "hdy-dialer.h"
-#include "hdy-dialog.h"
+#include "hdy-deck.h"
+#include "hdy-deprecation-macros.h"
 #include "hdy-enum-value-object.h"
 #include "hdy-expander-row.h"
-#include "hdy-fold.h"
+#include "hdy-header-bar.h"
 #include "hdy-header-group.h"
+#include "hdy-keypad.h"
 #include "hdy-leaflet.h"
-#include "hdy-list-box.h"
 #include "hdy-main.h"
+#include "hdy-navigation-direction.h"
+#include "hdy-preferences-group.h"
+#include "hdy-preferences-page.h"
+#include "hdy-preferences-row.h"
+#include "hdy-preferences-window.h"
 #include "hdy-search-bar.h"
-#include "hdy-string-utf8.h"
+#include "hdy-squeezer.h"
+#include "hdy-swipe-group.h"
+#include "hdy-swipe-tracker.h"
+#include "hdy-swipeable.h"
 #include "hdy-title-bar.h"
+#include "hdy-types.h"
 #include "hdy-value-object.h"
+#include "hdy-view-switcher.h"
+#include "hdy-view-switcher-bar.h"
+#include "hdy-view-switcher-title.h"
+#include "hdy-window.h"
+#include "hdy-window-handle.h"
 
 #undef _HANDY_INSIDE
 
